!{\src2tex{textfont=tt}}
!!****m* ABINIT/m_wannier2abinit
!! NAME
!!  m_wannier2abinit
!!
!! FUNCTION
!!
!! COPYRIGHT
!! Copyright (C) 2008-2009 ABINIT group (MG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!! 
!! OUTPUT
!!
!! PARENTS 
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

!@ ABIDOC
MODULE m_wannier2abinit

 use defs_basis
 use defs_datatypes
 use m_errors,       only : assert
 use m_geometry,     only : wigner_seitz, normv
 use m_io_tools,     only : get_unit

 implicit none

! === List of available public routines and functions ===
 public ::         &
&  InitWanData,    & ! Creation method.
&  DestroyWanData, & ! Destruction method.
&  DumpWanData,    & ! Dump the object on file.
&  ReadWanData,    & ! Read the object from file.
&  MakeWannierHR,  & ! Construct matrix elements of H in the Wannier representation.
&  PlotWannierHR,  & ! Plot the above matrix elements.
&  PrintWanData,   & ! Printout of the content of the object
&  WanMatInterpol, & ! Public interface to interpolate matrix elements.
&  WrapInitWanData   ! Wrapper for the creation method, needed to bypass the problem with nsppol=2
                     ! Waiting for a better integration of Wannier90 and improvements of the library-mode

!@END ABIDOC

 interface WanMatInterpol
  module procedure WanMatInterpol_dpc
  module procedure WanMatInterpol_dp
 end interface 

 PRIVATE

 integer,parameter :: LATEST_WD_VERSION=0
 complex(dpc),parameter :: j_dpc=(0.0_dp,1.0_dp)

 integer,parameter :: LMAX(3)=(/2,2,2/)
 real(dp),parameter :: gamma_point(3)=(/zero,zero,zero/)

CONTAINS  !===========================================================
!!***

!!****f* m_wannier2abinit/InitWanData
!! NAME
!! InitWanData
!!
!! FUNCTION
!!  Creation method for the WannierData object.
!!  The most important results of wannier90 are passed in input to the routine which
!!  allocates and initializes the structure.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      m_wannier2abinit
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE

subroutine InitWanData(WanData,mband,nkpt,nsppol,nntot,mwan,nwan,have_disentangled,eigen,&
& band_in,lwindow,wann_centres,wann_spreads,spreadw,U_matrix,U_matrix_opt,Hdr,W90version,title)

 use defs_basis
 use m_errors, only : assert_eq

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_14iowfdenpot
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nkpt,mband,nntot,nsppol,mwan
 real(dp),intent(in) :: W90version
 type(Hdr_type),intent(in) :: Hdr
 type(WannierData),intent(out) :: WanData
 character(len=*),intent(in) :: title
!arrays
 integer,intent(in) :: nwan(nsppol)
 real(dp),intent(in) :: spreadw(3,nsppol),wann_centres(3,mwan,nsppol),wann_spreads(mwan,nsppol)
 real(dp),intent(in) :: eigen(mband*nkpt*nsppol)
 complex(dpc),intent(in) :: U_matrix(mwan,mwan,nkpt,nsppol)
 !complex(dpc),intent(in) :: U_matrix_opt(nwan,nwan,nkpt)
 complex(dpc),intent(in) :: U_matrix_opt(:,:,:,:)
 logical,intent(in) :: have_disentangled(nsppol)
 logical,intent(in) :: band_in(mband,nsppol)
 logical,intent(in) :: lwindow(mband,nkpt,nsppol)

!Local variables---------------------------
!scalars
 integer :: ikpt,isp,nband_k,idx,iband,ii
! *************************************************************************

 call NullifyWanData_(WanData)

 WanData%mband     =mband
 WanData%mwan      =mwan
 WanData%nntot     =nntot
 WanData%nkpt      =nkpt
 WanData%nsppol    =nsppol
 WanData%W90version=W90version
 WanData%WDversion =LATEST_WD_VERSION

 WanData%title=title

 ! === Arrays ===
 allocate(WanData%nwan(nsppol))
 WanData%nwan=nwan

 allocate(WanData%eigen(mband,nkpt,nsppol))
 idx=0
 do isp=1,nsppol
  do ikpt=1,nkpt
   nband_k=Hdr%nband(ikpt+(isp-1)*nkpt)
   do iband=1,nband_k
    idx=idx+1
    WanData%eigen(iband,ikpt,isp)=eigen(idx)
   end do
  end do
 end do

 allocate(WanData%spreadw(3,nsppol))
 WanData%spreadw=spreadw

 allocate(WanData%wann_centres(3,mwan,nsppol))
 WanData%wann_centres=wann_centres

 allocate(WanData%wann_spreads(mwan,nsppol))
 WanData%wann_spreads=wann_spreads

 allocate(WanData%U_matrix(mwan,mwan,nkpt,nsppol))
 WanData%U_matrix=U_matrix

 allocate(WanData%have_disentangled(nsppol))
 WanData%have_disentangled=have_disentangled

 allocate(WanData%lwindow(mband,nkpt,nsppol)) !TODO this should be done only if disentanglement
 WanData%lwindow=lwindow(:,:,:)

 if (ANY(have_disentangled)) then 
  ii = assert_eq(mband, SIZE(U_matrix_opt,DIM=1),'Wrong mband size',__FILE__,__LINE__) 
  ii = assert_eq(mwan,  SIZE(U_matrix_opt,DIM=2),'Wrong nwan size',__FILE__,__LINE__) 
  ii = assert_eq(nkpt,  SIZE(U_matrix_opt,DIM=3),'Wrong nkpt size',__FILE__,__LINE__) 
  ii = assert_eq(nsppol,SIZE(U_matrix_opt,DIM=4),'Wrong nsppol size',__FILE__,__LINE__) 
  allocate(WanData%ndimwin(nkpt,nsppol))
  do isp=1,nsppol
   do ikpt=1,nkpt
    WanData%ndimwin(ikpt,isp)=COUNT(WanData%lwindow(:,ikpt,isp))
   end do
  end do
  !TODO initialize U_matrix_opt if one of the spin has not been disentangled
  allocate(WanData%U_matrix_opt(mband,mwan,nkpt,nsppol))
  WanData%U_matrix_opt=U_matrix_opt
 end if

 allocate(WanData%band_in(mband,nsppol))
 WanData%band_in=band_in

 call hdr_copy(Hdr,WanData%Hdr)

end subroutine InitWanData
!!***

!!****f* m_wannier2abinit/WrapInitWanData
!! NAME
!! WrapInitWanData
!!
!! FUNCTION
!!  Just a wrapper for the true creation method to avoid problems with the extra dimension
!!  allocated for the spin. Wannier90, for the moment, accepts only a single spin for run!
!!  The WannierData, instead, takes into account this possibility.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      mlwfovlp
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE
subroutine WrapInitWanData(WanData,mband,nkpt,nsppol,nntot,mwan,nwan_W,have_disentangled_W,eigen,&
& band_in_W,lwindow_W,wann_centres_W,wann_spreads_W,spreadw_W,U_matrix_W,U_matrix_opt_W,Hdr,W90version,title)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nkpt,mband,nntot,nsppol,mwan
 real(dp),intent(in) :: W90version
 type(Hdr_type),intent(in) :: Hdr
 type(WannierData),intent(out) :: WanData
 character(len=*),intent(in) :: title
!arrays
 integer,intent(in) :: nwan_W
 real(dp),intent(in) :: spreadw_W(3)
 real(dp),intent(in) :: wann_centres_W(3,mwan) 
 real(dp),intent(in) :: wann_spreads_W(mwan)
 real(dp),intent(in) :: eigen(mband*nkpt*nsppol)
 complex(dpc),intent(in) :: U_matrix_W(mwan,mwan,nkpt)
 complex(dpc),intent(in) :: U_matrix_opt_W(mband,mwan,nkpt)
 logical,intent(in) :: have_disentangled_W
 logical,intent(in) :: band_in_W(mband)
 logical,intent(in) :: lwindow_W(mband,nkpt)

!Local variables---------------------------
! Used to call the correct creation method
 integer :: nwan(nsppol)
 real(dp) :: spreadw(3,nsppol),wann_centres(3,mwan,nsppol),wann_spreads(mwan,nsppol)
 complex(dpc) :: U_matrix(mwan,mwan,nkpt,nsppol)
 complex(dpc) :: U_matrix_opt(mband,mwan,nkpt,nsppol)
 logical :: have_disentangled(nsppol)
 logical :: band_in(mband,nsppol)
 logical :: lwindow(mband,nkpt,nsppol)

! *************************************************************************

 nwan(:)              =nwan_W
 spreadw(:,:)         =SPREAD(spreadw_W(:),DIM=2,NCOPIES=nsppol) 
 wann_centres(:,:,:)  =SPREAD(wann_centres_W(:,:),DIM=3,NCOPIES=nsppol)
 wann_spreads(:,:)    =SPREAD(wann_spreads_W(:),DIM=2,NCOPIES=nsppol)
 U_matrix(:,:,:,:)    =SPREAD(U_matrix_W(:,:,:),DIM=4,NCOPIES=nsppol)
 U_matrix_opt(:,:,:,:)=SPREAD(U_matrix_opt_W(:,:,:),DIM=4,NCOPIES=nsppol)
 have_disentangled(:) =have_disentangled_W
 band_in(:,:)         =SPREAD(band_in_W(:)  ,DIM=2,NCOPIES=nsppol)
 lwindow(:,:,:)       =SPREAD(lwindow_W(:,:),DIM=3,NCOPIES=nsppol)

 call InitWanData(WanData,mband,nkpt,nsppol,nntot,mwan,nwan,have_disentangled,eigen,&
& band_in,lwindow,wann_centres,wann_spreads,spreadw,U_matrix,U_matrix_opt,Hdr,W90version,title)

end subroutine WrapInitWanData
!!***

!!****f* m_wannier2abinit/NullifyWanData_
!! NAME
!! NullifyWanData_  
!!
!! FUNCTION
!!  Nullify the pointers [Private]
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      m_wannier2abinit
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE
subroutine NullifyWanData_(WanData)


 implicit none

!Arguments ------------------------------------
 type(WannierData),intent(inout) :: WanData
! *************************************************************************

 nullify(WanData%nwan        )

 nullify(WanData%eigen       )
 nullify(WanData%spreadw     )
 nullify(WanData%wann_centres)
 nullify(WanData%wann_spreads)

 nullify(WanData%U_matrix    )
 nullify(WanData%U_matrix_opt)

 nullify(WanData%band_in     )
 nullify(WanData%lwindow     )
 nullify(WanData%have_disentangled)

 !TODO
 !call nullify(WanData%Hdr)

 ! === Variables used to interpolate
 nullify(WanData%irvec  )
 nullify(WanData%ndegen )
 nullify(WanData%ndimwin)
 nullify(WanData%hamWR  )

end subroutine NullifyWanData_
!!***

!!****f* m_wannier2abinit/DestroyWanData
!! NAME
!! DestroyWanData
!!
!! FUNCTION
!! Free the dynamic memory in the object.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      gw_tools,mlwfovlp
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE
subroutine DestroyWanData(WanData)
 
 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_14iowfdenpot
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(WannierData),intent(inout) :: WanData
! *************************************************************************

 if (associated(WanData%nwan             ))  deallocate(WanData%nwan             )
 if (associated(WanData%eigen            ))  deallocate(WanData%eigen            )
 if (associated(WanData%spreadw          ))  deallocate(WanData%spreadw          )
 if (associated(WanData%wann_centres     ))  deallocate(WanData%wann_centres     )
 if (associated(WanData%wann_spreads     ))  deallocate(WanData%wann_spreads     )

 if (associated(WanData%U_matrix         ))  deallocate(WanData%U_matrix         )
 if (associated(WanData%U_matrix_opt     ))  deallocate(WanData%U_matrix_opt     )

 if (associated(WanData%band_in          ))  deallocate(WanData%band_in          )
 if (associated(WanData%lwindow          ))  deallocate(WanData%lwindow          )
 if (associated(WanData%have_disentangled))  deallocate(WanData%have_disentangled)

 call hdr_clean(WanData%Hdr)

 ! === Variables used to interpolate
 if (associated(WanData%irvec  )) deallocate(WanData%irvec  )
 if (associated(WanData%ndegen )) deallocate(WanData%ndegen )
 if (associated(WanData%ndimwin)) deallocate(WanData%ndimwin)
 if (associated(WanData%hamWR  )) deallocate(WanData%hamWR  )

end subroutine DestroyWanData
!!***

!!****f* m_wannier2abinit/DumpWanData
!! NAME
!! DumpWanData
!!
!! FUNCTION
!! Dump the object on an external binary file.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      mlwfovlp
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE

subroutine DumpWanData(WanData,fname,accesswff)

 use defs_basis
 use m_io_tools, only : get_unit

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_13io_mpi
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: accesswff 
 !character(len=*)  This might create problems with other abinit routines!
 character(len=fnlen),intent(in) :: fname
 type(WannierData),intent(inout) :: WanData

!Local variables---------------------------
!scalars
 integer :: unt,fform,rdwr,isp
! *************************************************************************

 fform=502 ; rdwr=2
 ! Add date and time?

 select case (accesswff)
 case (0)
   unt=get_unit()
   open(unt,file=fname,form='unformatted')
   
   ! === Write the (in)famous Abinit Header
   call hdr_io(fform,WanData%Hdr,rdwr,unt)

   ! === Info ===
   write(unt)WanData%title, WanData%W90version, WanData%WDversion

   ! === Dimensions and parameters of the calculation  ===
   write(unt)WanData%mband, WanData%nkpt, WanData%nntot, WanData%nsppol, WanData%mwan
   do isp=1,WanData%nsppol
    write(unt)WanData%nwan(isp),WanData%have_disentangled(isp)
    write(unt)WanData%band_in(:,isp) 
    write(unt)WanData%lwindow(:,:,isp) !this is only for disentangled!
   end do

   do isp=1,WanData%nsppol
    ! * Energies
    write(unt)WanData%eigen(:,:,isp)
    ! * Results for Wan spread.
    write(unt)WanData%spreadw(:,isp), WanData%wann_centres(:,:,isp), WanData%wann_spreads(:,isp)

    write(unt)WanData%U_matrix(:,:,:,isp)
    if (WanData%have_disentangled(isp)) then 
     write(unt)WanData%U_matrix_opt(:,:,:,isp)
    end if
   end do

   close(unt)

 case default
  stop 'Wrong accesswff or not implemented error'
 end select 

end subroutine DumpWanData
!!***

!!****f* m_wannier2abinit/ReadWanData
!! NAME
!! ReadWanData
!!
!! FUNCTION
!! Read the object from file.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      gw_tools
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE

subroutine ReadWanData(WanData,fname,accesswff)

 use defs_basis
 use m_io_tools, only : get_unit

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_13io_mpi
 use interfaces_14iowfdenpot
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: accesswff 
 !character(len=*)  This might create problems with other abinit routines!
 character(len=fnlen),intent(in) :: fname
 type(WannierData),intent(out) :: WanData

!Local variables---------------------------
!scalars
 integer :: iband,ikpt,isp,nband_k,unt,fform,rdwr,idx,mwan
 integer :: mband,nntot,nkpt,nsppol,WDversion
 real(dp) :: W90version
 character(len=800) :: title
 type(Hdr_type) ::  Hdr
!Arrays 
 integer,allocatable ::  nwan(:)
 real(dp),allocatable :: spreadw(:,:)
 real(dp),allocatable :: wann_centres(:,:,:),wann_spreads(:,:),eigen(:,:,:),eigpack(:)
 logical,allocatable ::  band_in(:,:),lwindow(:,:,:),have_disentangled(:)
 complex(dp),allocatable :: U_matrix(:,:,:,:),U_matrix_opt(:,:,:,:) 
! *************************************************************************

 rdwr=1 ! Add date and time?

 select case (accesswff)
 case (0)
   unt=get_unit()
   open(unt,file=fname,form='unformatted')
   
   ! === Read the (in)famous Abinit Header
   call hdr_io(fform,Hdr,rdwr,unt)

   ! === Read Info ===
   read(unt)title, W90version, WDversion

   ! === Read Dimensions and parameters of the calculation  ===
   read(unt)mband, nkpt, nntot, nsppol, mwan

   allocate(nwan(nsppol),have_disentangled(nsppol))
   allocate(band_in(mband,nsppol),lwindow(mband,nkpt,nsppol))
   
   do isp=1,nsppol
    read(unt)nwan(isp), have_disentangled(isp)
    read(unt)band_in(:,isp) 
    read(unt)lwindow(:,:,isp) !this is only for disentangled!
   end do

   allocate(eigen(mband,nkpt,nsppol))
   allocate(spreadw(3,nsppol),wann_centres(3,mwan,nsppol),wann_spreads(mwan,nsppol))
   allocate(U_matrix(mwan,mwan,nkpt,nsppol))
   if (ANY(have_disentangled)) then
    allocate(U_matrix_opt(mband,mwan,nkpt,nsppol))
   end if

   do isp=1,nsppol
    ! * Read Energies
    read(unt)eigen(:,:,isp)
    ! * Read Results for Wan spread.
    read(unt)spreadw(:,isp), wann_centres(:,:,isp), wann_spreads(:,isp)

    read(unt)U_matrix(:,:,:,isp)
    if (have_disentangled(isp)) then 
     read(unt)U_matrix_opt(:,:,:,isp)
    end if
   end do

   close(unt)

   ! I have to temporary pack the energies to follow the funny Abinit convention.
   ! TODO call pack_eneocc, once build-sytem will treat correctly dependencies.
   allocate(eigpack(mband*nkpt*nsppol))
   do isp=1,nsppol
    do ikpt=1,nkpt
     nband_k=Hdr%nband(ikpt+(isp-1)*nkpt)
     do iband=1,nband_k
      idx=idx+1
      eigpack(idx)=eigen(iband,ikpt,isp)
     end do
    end do
   end do

   call InitWanData(WanData,mband,nkpt,nsppol,nntot,mwan,nwan,have_disentangled,eigpack,&
&   band_in,lwindow,wann_centres,wann_spreads,spreadw,U_matrix,U_matrix_opt,Hdr,W90version,title)

   ! Brand-new WanData object, deallocate temporary stuff.
   deallocate(eigpack)

   call hdr_clean(Hdr)
   deallocate(nwan,band_in,lwindow)
   deallocate(eigen)
   deallocate(spreadw,wann_centres,wann_spreads)
   deallocate(U_matrix)
   if (ANY(have_disentangled)) deallocate(U_matrix_opt)
   deallocate(have_disentangled)

 case default
  stop 'Wrong accesswff or not implemented error'
 end select 

end subroutine ReadWanData
!!***

!!****f* m_wannier2abinit/PrintWanData
!! NAME
!! PrintWanData
!!
!! FUNCTION
!! Printout of the content of the object (useful for debugging)
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      gw_tools
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE

subroutine PrintWanData(WanData,header,unit,mode_paral,prtvol)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_13io_mpi
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 integer,optional,intent(in) :: unit,prtvol
 character(len=*),optional,intent(in) :: header
 character(len=4),optional,intent(in) :: mode_paral
 type(WannierData),intent(inout) :: WanData

!Local variables-------------------------------
!scalars
 integer :: rdwr,fform,unt,verbose,isp,ikpt
 character(len=4) :: mode
 character(len=500) :: msg

! *************************************************************************

 unt=std_out ; if (PRESENT(unit))       unt=unit
 verbose=0   ; if (PRESENT(prtvol))     verbose=prtvol
 mode='COLL' ; if (PRESENT(mode_paral)) mode=mode_paral


 ! TODO define fform and version, solve well known problems with build system
 rdwr=4 ; fform=502 ! Define fform for WAN HDR?
 call hdr_io(fform,WanData%Hdr,rdwr,unt) !this call requires INOUT intent.

 ! TODO cleaning and beautification
 write(unt,*)' Wannier90 version ',WanData%W90version 
 write(unt,*)' Version of the Wannier data file ',WanData%WDversion 

 write(unt,*)' Maximun number of bands processed    ',WanData%mband
 write(unt,*)' Number of Wannier functions  ',WanData%nwan(:)
 write(unt,*)' Number of spin polarizations    ',WanData%nsppol
 write(unt,*)' Number of kpoints and neighbours ',WanData%nkpt,WanData%nntot

 write(unt,*)' Have disentangled ',WanData%have_disentangled(:)
 write(unt,*)' spreadw ',WanData%spreadw     
 write(unt,*)' Wannier centres ',WanData%wann_centres
 write(unt,*)' Wannier spreads ',WanData%wann_spreads

 do isp=1,WanData%nsppol
  write(unt,*)' For spin ',isp,COUNT(WanData%band_in(:,isp)),' bands are included '
  if (WanData%have_disentangled(isp)) then
   do ikpt=1,WanData%nkpt
    write(unt,*)' For spin ',isp,' k-point ',ikpt,COUNT(WanData%lwindow(:,ikpt,isp)),' bands are included'
   end do
  end if
 end do

 if (verbose>0) then
  do isp=1,WanData%nsppol
   write(unt,*)' Energies ',WanData%eigen(:,:,isp)
   !write(unt,*)' U matrix ',WanData%U_matrix(:,:,:,isp)
   if (WanData%have_disentangled(isp)) then
    !write(unt,*)'U_matrix_opt',WanData%U_matrix_opt(:,:,:,isp)
   end if
  end do
 end if

end subroutine PrintWanData
!!***

!!****f* m_wannier2abinit/MakeWannierHR
!! NAME
!! MakeWannierHR
!!
!! FUNCTION
!! Construct the matrix elements of the Hamiltonian in real space.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! TODO 
!! kptrlatt is passed in input but it should be reported in the abinit header
!! as well as shiftk and kptopt
!! Add additional options to cut the matrix.
!!
!! PARENTS
!!      gw_tools
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE

subroutine MakeWannierHR(WanData,kptrlatt)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(WannierData),target,intent(inout) :: WanData
!scalars
 integer,intent(in) :: kptrlatt(3,3)

!Local variables-------------------------------
!scalars
 integer :: isp,ikpt,idx,ib,nwan,nkpt,mband,ii,jj,kk,iR,mwan,nrpts
 real(dp) :: rdotk,ucvol
 complex(dpc) :: phase
 character(len=500) :: msg
 logical :: ass_irvec,ass_ndegen
!Arrays 
 real(dp) :: irvec_tmp(3)
 integer,allocatable :: shift_vec(:,:)
 integer,pointer :: irvec(:,:),ndegen(:)
 real(dp) :: rmet(3,3)
 real(dp),pointer :: kpt(:,:),rprimd(:,:)
 real(dp),allocatable :: eigval_opt(:,:)
 real(dp),allocatable :: eigval2(:,:)
 complex(dpc),pointer :: U_matrix(:,:,:),U_matrix_opt(:,:,:)
 complex(dpc),allocatable :: ham_k(:,:,:),ham_r(:,:,:)

! *************************************************************************

 ! === If not yet done calculate Wigner-Seitz cell ===
 ! * Here we should consider the possibility of a different real mesh.
 ass_irvec =associated(WanData%irvec )
 ass_ndegen=associated(WanData%ndegen)

 if (.not.ass_irvec.or..not.ass_ndegen) then
  call assert((.not.ass_irvec ),'irvec  should not be associated',__FILE__,__LINE__)
  call assert((.not.ass_ndegen),'ndegen should not be associated',__FILE__,__LINE__)
  rprimd => WanData%Hdr%rprimd
  do ii=1,3 ! Compute real space metrics
   rmet(ii,:)=rprimd(1,ii)*rprimd(1,:)+&
&             rprimd(2,ii)*rprimd(2,:)+&
&             rprimd(3,ii)*rprimd(3,:)
  end do

  call wigner_seitz(gamma_point,LMAX,kptrlatt,rmet,nrpts,irvec,ndegen,prtvol=0)
  WanData%nrpts=nrpts
  allocate(WanData%irvec(3,nrpts)) ; WanData%irvec =irvec
  allocate(WanData%ndegen(nrpts))  ; WanData%ndegen=ndegen
  deallocate(irvec,ndegen)
 end if

 ! === Allocate Hamiltonian in the Wannier gauge on the ab initio grid ===
 ! * The Max number of Wan bands over spin is used to allocate sufficient space.

 mband=WanData%mband
 nkpt =WanData%nkpt
 mwan =WanData%mwan 
 kpt => WanData%Hdr%kptns(1:3,1:nkpt)

 if (associated(WanData%hamWR)) deallocate(WanData%hamWR)
 allocate(WanData%hamWR(mwan,mwan,WanData%nrpts,WanData%nsppol))
 WanData%hamWR=czero

 do isp=1,WanData%nsppol
  nwan =WanData%nwan(isp)

  allocate(ham_k(nwan,nwan,nkpt))
  ham_k=czero

  allocate(eigval2(nwan,nkpt))
  eigval2=zero

  if (WanData%have_disentangled(isp)) then
   !
   ! === Slim down eigval to contain states within the outer window ===
   allocate(eigval_opt(mband,nkpt))
   eigval_opt=zero
   do ikpt=1,WanData%nkpt
    idx=0
    do ib=1,WanData%mband
     if (WanData%lwindow(ib,ikpt,isp)) then
      idx=idx+1
      eigval_opt(idx,ikpt)=WanData%eigen(ib,ikpt,isp)
     end if
    end do
   end do
   
   ! === Rotate eigval into the optimal subspace ===
   ! * In general eigval would be a matrix at each kpoint
   !   but we choose u_matrix_opt such that the Hamiltonian is
   !   diagonal at each kpoint. (I guess we should check it here)
   u_matrix_opt => WanData%U_matrix_opt(:,:,:,isp)
   
   do ikpt=1,WanData%nkpt
    do jj=1,nwan
     do kk=1,WanData%ndimwin(ikpt,isp)
      eigval2(jj,ikpt)=eigval2(jj,ikpt)+ & 
&     eigval_opt(kk,ikpt)*real(conjg(u_matrix_opt(kk,jj,ikpt))*u_matrix_opt(kk,jj,ikpt),dp)
     end do
    end do
   end do
   deallocate(eigval_opt)

  else
   !FIXME here there is a bug if we dont start from 1
   eigval2(1:nwan,:)=WanData%eigen(1:nwan,:,isp)
  end if

  ! At this point eigval2 contains nwan values belonging to the Wannier subspace.
  ! Rotate the Hamiltonian into the basis of smooth Bloch states.
  !          H(k)=U^{dagger}(k).H_0(k).U(k)
  ! Note: we enforce hermiticity here
  u_matrix => WanData%U_matrix(:,:,:,isp)

  do ikpt=1,WanData%nkpt
   do jj=1,nwan
    do ii=1,jj
     do kk=1,nwan
      ham_k(ii,jj,ikpt) = ham_k(ii,jj,ikpt)+ &
&      eigval2(kk,ikpt)*CONJG(u_matrix(kk,ii,ikpt))*u_matrix(kk,jj,ikpt)
     end do
     if (ii<jj) ham_k(jj,ii,ikpt)=CONJG(ham_k(ii,jj,ikpt))
    end do
   end do
  end do

  deallocate(eigval2)

  ! === Fourier transform rotated hamiltonian into WF basis ===
  ! * H_ij(k) --> H_ij(R) = (1/N_kpts) sum_k e^{-ikR} H_ij(k)
  allocate(ham_r(nwan,nwan,WanData%nrpts))
  ham_r=czero

  !TODO
  !if (.not.use_translation) then
  if (.TRUE.) then
   do iR=1,WanData%nrpts
    do ikpt=1,WanData%nkpt
     rdotk=two_pi*DOT_PRODUCT(kpt(:,ikpt),real(WanData%irvec(:,iR),dp))
     phase=EXP(-j_dpc*rdotk) !/real(num_kpts,dp)
     ham_r(:,:,iR)=ham_r(:,:,iR)+phase*ham_k(:,:,ikpt)
    end do
   end do
   ham_r=ham_r/WanData%nkpt
   !have_translated = .false.

  else
   allocate(shift_vec(3,nwan))
   stop "not implemented" !TODO
   !call internal_translate_centres()
   do iR=1,nrpts
    do ikpt=1,WanData%nkpt
     do ii=1,nwan
      do jj=1,nwan
       ! ham_r(j,i,iR) interaction btw j at 0 and i at irvec(:,iR)
       irvec_tmp(:)=WanData%irvec(:,iR)+shift_vec(:,ii)-shift_vec(:,jj)   
       rdotk=two_pi*DOT_PRODUCT(kpt(:,ikpt),real(irvec_tmp(:),dp))
       phase=EXP(-j_dpc*rdotk) !/real(num_kpts,dp)
       ham_r(jj,ii,iR)=ham_r(jj,ii,iR)+phase*ham_k(jj,ii,ikpt)
      end do
     end do
    end do
   end do
   ham_r=ham_r/WanData%nkpt
   !have_translated = .true.
   deallocate(shift_vec)
  end if

  ! === Save Hamiltonian in real space ===
  WanData%hamWR(1:nwan,1:nwan,:,isp)=ham_r(:,:,:)
  deallocate(ham_r)
 end do !isp

end subroutine MakeWannierHR
!!***

!!****f* m_wannier2abinit/PlotWannierHR
!! NAME
!! PlotWannierHR
!!
!! FUNCTION
!!  Plot the matrix elements in real space of the Hamitonian in the Wannier Gauge.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE

subroutine PlotWannierHR(WanData,fname,cut_mode)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_lib03numeric
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 character(len=*),intent(in) :: fname
 character(len=*),optional,intent(in) :: cut_mode
 type(WannierData),target,intent(inout) :: WanData
!scalars

!Local variables-------------------------------
!scalars
 integer :: unt,iR,irad,isp
 integer :: nwan,ii,jj,mwan,nrpts,ilen
 real(dp) :: datum(WanData%nsppol)
 character(len=500) :: msg
!Arrays 
 real(dp) :: irvec_tmp(3)
 integer,allocatable :: shift_vec(:,:),sph2arr(:)
 integer,pointer :: irvec(:,:),ndegen(:)
 real(dp) :: rmet(3,3)
 real(dp),pointer :: rprimd(:,:)
 real(dp),allocatable :: length(:)

! *************************************************************************

 if (PRESENT(cut_mode)) then
  if (WanData%cut_mode/=cut_mode) STOP 'not implemented'
 end if

 rprimd => WanData%Hdr%rprimd
 do ii=1,3 ! Compute real space metrics
  rmet(ii,:)=rprimd(1,ii)*rprimd(1,:)+&
&            rprimd(2,ii)*rprimd(2,:)+&
&            rprimd(3,ii)*rprimd(3,:)
 end do

 allocate(length(WanData%nrpts),sph2arr(WanData%nrpts))
 do iR=1,WanData%nrpts
  length(iR)=normv(WanData%irvec(:,iR),rmet,'R')
  sph2arr(iR)=iR
 end do

 call sort_dp(WanData%nrpts,length,sph2arr,tol6)
 deallocate(length)

 unt=get_unit()
 open(file=fname,unit=unt,form='formatted')

 write(unt,'(a)')'# '
 write(unt,'(a)')'# Matrix elements of H in the Wannier representation '
 write(unt,'(a)')'# Length(ii), Lattice point Rws(1:3,ii), (MAXVAL |hamW_ij(R,isp)|, isp=1,nsppol)'
 write(unt,'(a)')'# '
 do irad=1,WanData%nrpts
  ii=sph2arr(irad)
  do isp=1,WanData%nsppol
   nwan=WanData%nwan(isp)
   datum(isp)=MAXVAL(ABS(WanData%hamWR(1:nwan,1:nwan,ii,isp)))
  end do
  write(unt,'(es14.6,3i3,(es14.6))')length(ii),WanData%irvec(:,ii),datum(:)
 end do

 close(unt)
 deallocate(sph2arr)

end subroutine PlotWannierHR
!!***


!!****f* m_wannier2abinit/WanMatInterpol_dpc
!! NAME
!! WanMatInterpol_dpc
!!
!! FUNCTION
!! Interpolate matrix elements on a denser k-mesh using Wannier representation
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! NOTES
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE

subroutine WanMatInterpol_dpc(WanData,nkab,nk4intp,k4intp,ishermitian,nsppol,mb4intp,matrix_in,matrix_out)

 use defs_basis
 use m_errors,  only : assert

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nk4intp,nkab,nsppol,mb4intp
 logical,intent(in) :: ishermitian
 type(WannierData),intent(in) :: WanData
!Arrays
 real(dp),intent(in) :: k4intp(3,nk4intp)
 complex(dpc),intent(in)  :: matrix_in (mb4intp,mb4intp,nkab,   nsppol)
 complex(dpc),intent(out) :: matrix_out(mb4intp,mb4intp,nk4intp,nsppol)

!Local variables-------------------------------
!scalars
 logical :: ltest
! *********************************************************************

 ! === Consistency check === 
 ltest=(WanData%nkpt==nkab)
 call assert(ltest,'WanData%nkpt/=nkab')
 ltest=(WanData%nsppol==nsppol)
 call assert(ltest,'WanData%nsppol/=nsppol')
 ltest=(mb4intp==WanData%mwan)
 call assert(ltest,'WanData%mwan/=mb4intp')

 ! === Call the private method to do the dirty job ===
 call DoWanMatInterpol_(WanData,nk4intp,k4intp,ishermitian,matrix_in,matrix_out)

end subroutine WanMatInterpol_dpc
!!***

!!****f* ABINIT/WanMatInterpol_dp
!! NAME
!! WanMatInterpol_dp
!!
!! FUNCTION
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE
subroutine WanMatInterpol_dp(WanData,nkab,nk4intp,k4intp,ishermitian,nsppol,mb4intp,matrix_in,matrix_out)

 use defs_basis
 use m_errors,  only : assert

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nk4intp,nkab,nsppol,mb4intp
 logical,intent(in) :: ishermitian
 type(WannierData),intent(in) :: WanData
!Arrays
 real(dp),intent(in) :: k4intp(3,nk4intp)
 real(dp),intent(in)  :: matrix_in (mb4intp,mb4intp,nkab,   nsppol)
 real(dp),intent(out) :: matrix_out(mb4intp,mb4intp,nk4intp,nsppol)

!Local variables-------------------------------
!scalars
 logical :: ltest
!Arrays
 complex(dpc) :: c_matrix_in (mb4intp,mb4intp,nkab,   nsppol)
 complex(dpc) :: c_matrix_out(mb4intp,mb4intp,nk4intp,nsppol)
! *********************************************************************

 ! === Consistency check === 
 ltest=(WanData%nkpt==nkab)
 call assert(ltest,'WanData%nkpt/=nkab')
 ltest=(WanData%nsppol==nsppol)
 call assert(ltest,'WanData%nsppol/=nsppol')
 ltest=(mb4intp==WanData%mwan)
 call assert(ltest,'WanData%mwan/=mb4intp')
 call assert(ishermitian,'ishermitian should be true for the real version')

 ! === Transfer input data to complex array ===
 c_matrix_in=CMPLX(matrix_in)

 ! === Call the private method to do the dirty job ===
 call DoWanMatInterpol_(WanData,nk4intp,k4intp,ishermitian,c_matrix_in,c_matrix_out)

 matrix_out=REAL(c_matrix_out)

end subroutine WanMatInterpol_dp
!!***

!!****f* m_wannier2abinit/DoWanMatInterpol_
!! NAME
!! DoWanMatInterpol_ [PRIVATE]
!!
!! FUNCTION
!! Interpolate matrix elements on a denser k-mesh using Wannier representation
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      m_wannier2abinit
!!
!! NOTES
!!  Do not use this method directly (it is private). 
!!  Use instead the public wrapper 
!!
!! CHILDREN
!!      zhpevx
!!
!! SOURCE


subroutine DoWanMatInterpol_(WanData,nk4intp,k4intp,ishermitian,matrix_in,matrix_out)

 use defs_basis
 !use m_numeric_tools, only : get_diag

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_lib03numeric
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nk4intp
 logical,intent(in) :: ishermitian
 type(WannierData),target,intent(in) :: WanData
!Arrays
 real(dp),intent(in) :: k4intp(3,nk4intp)
 complex(dpc),target,intent(in)  :: matrix_in (WanData%mwan,WanData%mwan,WanData%nkpt,WanData%nsppol)
 complex(dpc),target,intent(out) :: matrix_out(WanData%mwan,WanData%mwan,nk4intp,     WanData%nsppol)

!Local variables-------------------------------
!scalars
 integer :: ikab,ikint,nwan,nrpts,nrpts_cut,isp
 integer :: ierr,iR,ii,jj,kk,ll,info,nfound
 real(dp) :: rdotk,ABSTOL
 complex(dpc),parameter :: cmplx_i=(0.0_dp,1.0_dp)
 complex(dpc) :: fact,phase
 character(len=500) :: msg
!arrays
 integer,pointer :: irvec(:,:),ndegen(:)
 integer,allocatable :: iwork(:),ifail(:)
 real(dp),allocatable :: eig_interp(:,:),rwork(:)     
 complex(dpc),pointer :: Hwan_R(:,:,:),Ub2w(:,:,:),Ub2w_opt(:,:,:)
 complex(dpc),allocatable  :: Ham_r_cut(:,:,:),Ham_pack(:),Ham_kint(:,:),Owan_kint(:,:)
 complex(dpc),allocatable  :: Ub2w_intp(:,:),cwork(:),Owan_kab(:,:,:),Owan_R(:,:,:) 
 complex(dpc),pointer :: Oham_kab(:,:,:)
 !integer, allocatable :: irvec_cut(:,:)
 !integer              :: irvec_max(3)
 !integer              :: nrpts_cut
! *********************************************************************

 ! =====================================
 ! === Perform Wannier interpolation ===
 ! ======================================

 if (.not.associated(WanData%irvec).or..not.associated(WanData%ndegen)) then
  stop 'irvec and ndegen are not associated'
 end if

 nrpts  =  WanData%nrpts
 irvec  => WanData%irvec
 ndegen => WanData%ndegen
 
 matrix_out=czero

 do isp=1,WanData%nsppol

  ! === Rotate the Operator into the basis of smooth Bloch states ===
  ! * Owan(k)=U^*(k).0ham(k).U(k)

  nwan     =  WanData%nwan(isp) 
  Ub2w     => WanData%U_matrix(:,:,:,isp)
  Oham_kab => matrix_in(1:nwan,1:nwan,:,isp)

  allocate(Owan_kab(nwan,nwan,WanData%nkpt))
  Owan_kab=czero

  if (ishermitian) then 
   ! * Impose Hermiticity of Owan_kab
   do ikab=1,WanData%nkpt

    do jj=1,nwan
     do ii=1,jj
      do kk=1,nwan
       do ll=1,nwan
        Owan_kab(ii,jj,ikab) = Owan_kab(ii,jj,ikab)+ &
&        Oham_kab(kk,ll,ikab)*CONJG(Ub2w(kk,ii,ikab))*Ub2w(ll,jj,ikab)
       end do
      end do
      if (ii< jj) Owan_kab(jj,ii,ikab)=CONJG(Owan_kab(ii,jj,ikab))
      if (ii==jj) Owan_kab(jj,ii,ikab)=half*(Owan_kab(ii,jj,ikab)+CONJG(Owan_kab(ii,jj,ikab)))
     end do
    end do

   end do

  else
   do ikab=1,WanData%nkpt
    Owan_kab(:,:,ikab)= &
&    MATMUL( CONJG(TRANSPOSE(Ub2w(:,:,ikab))),MATMUL(Oham_kab(:,:,ikab),Ub2w(:,:,ikab)))
    !eigval2(kk,ikab)*CONJG(Ub2w(kk,ii,ikab))*Ub2w(kk,jj,ikab)
   end do
  end if

  ! === Fourier transform rotated hamiltonian into Wannier basis ===
  ! * H_ij(k) --> H_ij(R) = (1/N_kpts) sum_k e^{-ikR} H_ij(k)
  allocate(Owan_R(nwan,nwan,WanData%nrpts))
  Owan_R=czero

  do iR=1,WanData%nrpts
   do ikab=1,WanData%nkpt
    rdotk=two_pi*DOT_PRODUCT(WanData%Hdr%kptns(:,ikab),real(WanData%irvec(:,iR),dp))
    phase=EXP(-j_dpc*rdotk) 
    Owan_R(:,:,iR) = Owan_R(:,:,iR) + phase*Owan_kab(:,:,ikab)
   end do
  end do
  Owan_R=Owan_R/WanData%nkpt

  deallocate(Owan_kab)

  ! Pay attention here. Point only the Wannier functions for this spin i.e up to nwan.
  Hwan_R => WanData%hamWR(1:nwan,1:nwan,1:WanData%nrpts,isp)
  ! TODO add cut case at this level !if (cut) then ...

  allocate(eig_interp(nwan,nk4intp)) ! Interpolated energies, not used here.
  eig_interp=zero 

  allocate(Ham_pack((nwan*(nwan+1))/2),stat=ierr)
  allocate(Ham_kint(nwan,nwan),stat=ierr)
  allocate(Owan_kint(nwan,nwan),stat=ierr)
  allocate(Ub2w_intp(nwan,nwan),stat=ierr)
  allocate(cwork(2*nwan),rwork(7*nwan),iwork(5*nwan),ifail(nwan))
  !
  ! Cut H matrix in real-space
  !if (index(bands_plot_mode,'cut').ne.0)  call plot_cut_hr()

  ! === Interpolate the Hamiltonian at each kpoint 
  ! * Get Owan_kint on the interpolating grid then rotate to get Oham_kint. 

  do ikint=1,nk4intp

   Ham_kint(:,:)=czero
   Owan_kint(:,:)=czero
   !if (index(bands_plot_mode,'s-k').ne.0) then
    do iR=1,nrpts
     rdotk=two_pi*DOT_PRODUCT(k4intp(:,ikint),irvec(:,iR))
     fact=EXP(cmplx_i*rdotk)/REAL(ndegen(iR),dp)
     Ham_kint  =Ham_kint  +fact*Hwan_R(:,:,iR)
     Owan_kint =Owan_kint +fact*Owan_R(:,:,iR)
    end do
   !TODO
   !else if (index(bands_plot_mode,'cut').ne.0) then
   ! do iR=1,nrpts_cut
   !  rdotk=two_pi*DOT_PRODUCT(k4intp(:,ikint),irvec_cut(:,iR))
   !  !!$[aam] check divide by ndegen?
   !  fact=EXP(cmplx_i*rdotk)
   !  Ham_kint=Ham_kint+fact*Ham_r_cut(:,:,iR)
   !  Owan_kint =Owan_kint +fact*Owan_R(:,:,iR)
   ! end do
   !end if

   ! === Diagonalise H_k (->basis of eigenstates) ===
   do jj=1,nwan
    do ii=1,jj
     Ham_pack(ii+((jj-1)*jj)/2)=Ham_kint(ii,jj)
    end do
   end do

   ! TODO Do not need to allocate all the k-point in eig_interp
   ABSTOL=-one !; ABSTOL= 2*DLAMCH('S')
   call ZHPEVX('V','A','U',nwan,ham_pack,zero,zero,0,0,ABSTOL,&
    nfound,eig_interp(1,ikint),Ub2w_intp,nwan,cwork,rwork,iwork,ifail,info)

   if (info<0) then
    write(std_out,'(a,i3,a)') 'THE ',-info, ' ARGUMENT OF ZHPEVX HAD AN ILLEGAL VALUE'
   end if
   if (info>0) then
    write(std_out,'(i3,a)')info,' EIGENVECTORS FAILED TO CONVERGE'
   end if

   ! * Rotate and save interpolated matrix elements.
   if (ishermitian) then 
    do jj=1,nwan
     do ii=1,jj
      do kk=1,nwan
       do ll=1,nwan
        matrix_out(ii,jj,ikint,isp) = matrix_out(ii,jj,ikint,isp) + &
&        Owan_kint(kk,ll)*CONJG(Ub2w_intp(kk,ii))*Ub2w_intp(ll,jj)
       end do
      end do
      if (ii< jj) matrix_out(jj,ii,ikint,isp) = CONJG(matrix_out(ii,jj,ikint,isp))
      if (ii==jj) matrix_out(jj,ii,ikint,isp) = half*(matrix_out(ii,jj,ikint,isp)+CONJG(matrix_out(ii,jj,ikint,isp)))
     end do
    end do

   else 
    matrix_out(1:nwan,1:nwan,ikint,isp) = MATMUL( CONJG(TRANSPOSE(Ub2w_intp)),MATMUL(Owan_kint,Ub2w_intp))
   end if

   !write(77,'(7f8.4)')k4intp(:,ikint),eig_interp(1:nwan,ikint)

  end do !ikint

  ! Interpolation for this spin done.
  deallocate(eig_interp)
  deallocate(Ham_pack)
  deallocate(Ham_kint)
  deallocate(Owan_kint)
  deallocate(Ub2w_intp)
  deallocate(cwork,rwork,iwork,ifail)
  !if (allocated(Ham_r_cut)) deallocate(Ham_r_cut,stat=ierr)
  !if (allocated(irvec_cut)) deallocate(irvec_cut,stat=ierr)

 end do !isp

end subroutine DoWanMatInterpol_

END MODULE m_wannier2abinit
!!***
