!{\src2tex{textfont=tt}}
!!****f* ABINIT/setup_screening
!! NAME
!! setup_screening
!!
!! FUNCTION
!!  Initialize the Ep% data type containing the parameters used during the screening calculation.
!!  as well as basic objects describing the BZ sampling .... TODO list to be completed
!!
!! COPYRIGHT
!!  Copyright (C) 2008-2009 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!! acell(3)=length scales of primitive translations (Bohr).
!! Dtfil<Datafiles_type>=Variables related to files.
!! MPI_enreg<MPI_type>=Information about MPI parallelization.
!! rprim(3,3)=dimensionless real space primitive translations.
!! ngfftf(18)=Contain all needed information about the 3D FFT for densities and potentials.
!!
!! OUTPUT
!! ngfft_gw(18)=Contain all needed information about the 3D FFT for the oscillator strengths.
!!  See ~abinit/doc/input_variables/vargs.htm#ngfft
!! Ltg_q(:)<Little_group>,=
!! Ep<Epsilonm1_parameters>=Parameters for the screening calculation. 
!!  Most part of it is Initialized and checked.  
!! Hdr_kss type(Hdr_type)=Header of the KSS file.
!! Cryst<Crystal_structure>=Definition of the unit cell and its symmetries.
!! Kmesh<BZ_mesh_type>=Structure defining the k-point sampling (wavefunctions).
!! Qmesh<BZ_mesh_type>=Structure defining the q-point sampling (screening)
!! Gsph_wfn<Gvectors_type>=Structure defining the G-sphere for the wavefunctions (not k-dependent).
!! Gsph_epsG0<Gvectors_type>=The G-sphere for the screening, enlarged to take into account for umklapps.
!! Psps <Pseudopotential_type)>=Info on pseudopotential, only for consistency check of the KSS file 
!! Vcp <type Coulombian_type> datatype gathering information on the coulombian cutoff technique
!!
!! SIDE EFFECTS
!! Dtset<Dataset_type>=All input variables for this dataset.
!!  %ecutwfn, %npwwfn, %nshwfn,
!!  %ecuteps, %npweps, %nsheps
!!   might be redefinend in setshells in order to close the shell.
!!
!! NOTES
!!
!! PARENTS
!!      screening
!!
!! CHILDREN
!!      assert,bstruct_init,find_qmesh,flush,get_ng0sh,hdr_check,hdr_init
!!      hdr_update,hdr_vs_dtset,init_gvectors_type,initcrystalfromhdr,initkmesh
!!      leave_new,metric,mkrdim,nullify_epsilonm1_parameters,nullify_gvectors
!!      nullify_little_group,printcrystal,rhoij_alloc,rhoij_copy,rhoij_free
!!      setmesh,setshells,setup_coulombian,setup_little_group,testlda,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine setup_screening(codvsn,acell,rprim,ngfftf,Dtset,Dtfil,Psps,Pawtab,MPI_enreg,&
& ngfft_gw,Hdr_kss,Hdr_out,Cryst,Kmesh,Qmesh,KS_BSt,Ltg_q,Gsph_epsG0,Gsph_wfn,Vcp,Ep)
    
 use defs_basis
 use defs_datatypes
 use m_errors
 use m_bz_mesh
 use m_gwdefs,     only : GW_TOLQ0, GW_TOLQ, czero_gw
 use m_io_tools,   only : flush_unit
 use m_geometry,   only : normv
 use m_crystal
 use m_coulombian, only : setup_coulombian

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_01manage_mpi
 use interfaces_11util
 use interfaces_12geometry
 use interfaces_13recipspace
 use interfaces_14iowfdenpot
 use interfaces_15gw, except_this_one => setup_screening
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 character(len=6),intent(in) :: codvsn
 type(Dataset_type),intent(inout) :: Dtset !INOUT is due to setshells
 type(Datafiles_type),intent(in) :: Dtfil
 type(MPI_type),intent(inout) :: MPI_enreg !due to setup_coulombian
 type(Pseudopotential_type),intent(in) :: Psps
 type(Pawtab_type),intent(in) :: Pawtab(Psps%ntypat*Dtset%usepaw)
 type(Epsilonm1_parameters),intent(out) :: Ep 
 type(Hdr_type),intent(out) :: Hdr_kss,Hdr_out
 type(Bandstructure_type),intent(out) :: KS_BSt
 type(BZ_mesh_type),intent(out) :: Kmesh,Qmesh
 type(Crystal_structure),intent(out) :: Cryst
 type(Gvectors_type),intent(out) :: Gsph_epsG0,Gsph_wfn
 type(Coulombian_type),intent(out) :: Vcp
!arrays
 integer,intent(in) :: ngfftf(18)
 integer,intent(out) :: ngfft_gw(18)
 real(dp),intent(in) :: acell(3),rprim(3,3)
 type(Little_group),pointer :: Ltg_q(:)

!Local variables-------------------------------
!scalars
 integer,parameter :: NOMEGAGAUSS=30,NOMEGAREAL=201
 integer :: bantot,ib,ibtot,ikibz,io,iq,iqcalc,iqp,isppol,itypat 
 integer :: jj,mod10,nsym_kss,nbnds_kss,ng_kss
 integer :: mpsang_kss,timrev,mg0sh,use_umklp
 integer :: npwe_pG0,nsheps_pG0,method,enforce_sym,nfftgw_tot
 integer :: shift,pertcase_,restart,restartpaw
 real(dp),parameter :: OMEGAERMAX=100.0/Ha_eV
 real(dp) :: ecuteps_pG0,e0,ucvol
 logical :: remove_inv,ltest,only_one_kpt,found,is_static,has_q0
 character(len=500) :: msg      
!arrays
 integer :: ng0sh_opt(3)
 integer,allocatable :: npwarr(:),nlmn(:)
 integer,pointer :: gvec_p(:,:)
 real(dp) :: gmet(3,3),gprimd(3,3),rmet(3,3),rprimd(3,3),qtmp(3)
 real(dp),pointer :: energies_p(:,:,:)
 real(dp),allocatable :: doccde(:),eigen(:),occfact(:)
 type(Pawrhoij_type),allocatable :: Pawrhoij(:)
 
! *************************************************************************

 DBG_ENTER('COLL')

 ! === Check for calculations that are not implemented ===
 ltest=(Dtset%mkmem/=0)
 call assert(ltest,'mkmem=0 not yet implemented in screening.',__FILE__,__LINE__)
 ltest=ALL(Dtset%nband(1:Dtset%nkpt*Dtset%nsppol)==Dtset%nband(1))
 call assert(ltest,'Dtset%nband must be constant',__FILE__,__LINE__)

 call mkrdim(acell,rprim,rprimd)  
 call metric(gmet,gprimd,-1,rmet,rprimd,ucvol) 

 ! === Set up basic parameters of the calculation ===
 call nullify_epsilonm1_parameters(Ep) 

 Ep%gwcalctyp            =Dtset%gwcalctyp
 Ep%plasmon_pole_model   =.TRUE.  
 Ep%analytic_continuation=.FALSE. 
 Ep%contour_deformation  =.FALSE.

 mod10=MOD(Ep%gwcalctyp,10)
 if (mod10/=0.and.mod10/=8)            Ep%plasmon_pole_model=.FALSE.
 if (mod10==1)                         Ep%analytic_continuation=.TRUE.
 if (mod10==2.or.mod10==9)             Ep%contour_deformation=.TRUE.
 is_static=(mod10==5.or.mod10==6.or.mod10==7)

 Ep%nbnds  =Dtset%nband(1) 
 Ep%symchi =Dtset%symchi
 Ep%inclvkb=Dtset%inclvkb ; if (Dtset%usepaw/=0) Ep%inclvkb=0
 Ep%zcut   =Dtset%zcut   

 write(msg,'(2a,i4,2a,f10.6,a)')ch10,&
& ' GW calculation type              = ',Ep%gwcalctyp,ch10,&
& ' zcut to avoid poles in chi0 [eV] = ',Ep%zcut*Ha_eV,ch10
 call wrtout(std_out,msg,'COLL')
 !
 ! === Define consistently npw, nsh, and ecut for wavefunctions and dielectric matrix ===
 !now this part is done in invars2m
 !call setshells(Dtset%ecutwfn,Dtset%npwwfn,Dtset%nshwfn,Dtset%nsym,gmet,gprimd,Dtset%symrel,'wfn',ucvol)
 !call setshells(Dtset%ecuteps,Dtset%npweps,Dtset%nsheps,Dtset%nsym,gmet,gprimd,Dtset%symrel,'eps',ucvol)

 Ep%npwe  =Dtset%npweps 
 Ep%npwwfn=Dtset%npwwfn 
 Ep%npwvec=MAX(Ep%npwe,Ep%npwwfn)
 !
 ! === Read parameters from KSS and verifify them === 
 Ep%awtr=Dtset%awtr 
 call testlda(Dtset,Dtfil%filkss,Dtset%accesswff,Dtset%localrdwf,nsym_kss,&
& nbnds_kss,ng_kss,mpsang_kss,gvec_p,energies_p,Hdr_kss,MPI_enreg)

 ltest=(Psps%mpsang==mpsang_kss)
 call assert(ltest,'Psps%mpsang does not agree with mpsang read from KSS file',&
& __FILE__,__LINE__)

 ! === Get important dimension from Hdr_kss ===
 ! * Check also the consistency btw Hdr_kss and Dtset.
 Ep%nsppol=Hdr_kss%nsppol
 Ep%nkibz =Hdr_kss%nkpt

 call hdr_vs_dtset(Hdr_kss,Dtset) 
 remove_inv=(nsym_kss/=Hdr_kss%nsym) 

 timrev = 2 ! This information is not reported in the header
            ! 1 --> do not use time-reversal symmetry 
            ! 2 --> take advantage of time-reversal symmetry
 call InitCrystalFromHdr(Hdr_kss,Cryst,timrev,remove_inv)
 call PrintCrystal(Cryst,mode_paral='COLL')

 if (Ep%npwvec>ng_kss) then
  Ep%npwvec=ng_kss
  if (Ep%npwwfn> ng_kss) Ep%npwwfn=ng_kss
  if (Ep%npwe  > ng_kss) Ep%npwe  =ng_kss
  write(msg,'(5a,3(a,i6,a))')ch10,&
&  ' Number of G-vectors found less then required. ',ch10,&
&  ' Calculation will proceed with ',ch10,&
&  '  npwvec = ',Ep%npwvec,ch10,&
&  '  npweps = ',Ep%npwe  ,ch10,&
&  '  npwwfn = ',Ep%npwwfn,ch10
  MSG_WARNING(msg)
 end if

 if (Ep%nbnds>nbnds_kss) then
  Ep%nbnds=nbnds_kss
  Dtset%nband(:)=nbnds_kss
  write(msg,'(4a,i4,a)')ch10,&
&  ' Number of bands found less then required. ',ch10,&
&  ' Calculation will proceed with nbnds = ',nbnds_kss,ch10
  MSG_WARNING(msg)
 end if

 ! === Create basic data types for the calculation ===
 ! * Kmesh defines the k-point sampling for the wavefunctions.
 ! * Qmesh defines the q-point sampling for chi0, all possible differences k1-k2 reduced to the IBZ. 
 ! TODO Kmesh%bz should be [-half,half[ but this modification will be painful!

 call InitKmesh(Ep%nkibz,Hdr_kss%kptns,Cryst,Kmesh,Dtset%prtvol)
 !
 ! === Find Q-mesh, and do setup for long wavelength limit ===
 call find_Qmesh(Cryst,Kmesh,Qmesh,Dtset%prtvol)

 ! For the moment hard-coded values but they should be passed through Dtset%
 Ep%nqlwl=1 
 allocate(Ep%qlwl(3,Ep%nqlwl)) 
 Ep%qlwl(1,1)=0.000010
 Ep%qlwl(2,1)=0.000020
 Ep%qlwl(3,1)=0.000030

 ! === Find optimal value for G-sphere enlargment due to oscillator matrix elements ===
 mg0sh=25
 call get_ng0sh(Kmesh%nbz,Kmesh%bz,Qmesh%nibz,Qmesh%ibz,Kmesh%nbz,Kmesh%bz,gmet,GW_TOLQ0,mg0sh,ng0sh_opt)
 Ep%mG0(:)=ng0sh_opt(:)

 ! === In case of symmetrization, find the little group of the q"s ===
 ! * For the long-wavelength limit we consider a small but finite q. However the oscillators are 
 !  evaluated setting q==0. Thus it is possible to take advantage of symmetries also when q --> 0.
 ! * Here we calculate the enlargement of the G-sphere, npwepG0, needed to account for umklapps.
 ! TODO Switch on use_umklp, write all this stuff to ab_out

 Ep%npwepG0=Ep%npwe
 allocate(Ltg_q(Qmesh%nibz))
 do iq=1,Qmesh%nibz 
  call nullify_little_group(Ltg_q(iq)) 
  qtmp(:)=Qmesh%ibz(:,iq) ; if (normv(qtmp,gmet,'G')<GW_TOLQ0) qtmp(:)=zero ; use_umklp=0
  call setup_little_group(qtmp,Kmesh,Cryst,Ep%npwvec,gvec_p,Ep%npwe,use_umklp,Dtset%prtvol,Ltg_q(iq))
 end do

 if (Ep%symchi/=0) then
  ecuteps_pG0=MAXVAL(Ltg_q(:)%max_kin_gmG0)+tol6 ; npwe_pG0=0 ; nsheps_pG0=0
  write(*,*)' Due to umklapp processes : ecuteps_pg0=',ecuteps_pG0
  call setshells(ecuteps_pG0,npwe_pG0,nsheps_pG0,Cryst%nsym,gmet,gprimd,Cryst%symrel,'eps_pG0',Cryst%ucvol)
  Ep%npwepG0=npwe_pG0
 end if

 ltest=(Ep%npwepG0<=Ep%npwvec)
 write(msg,'(4a,i5,a,i5)')ch10,&
& ' npwepG0 > npwvec, decrease npweps or increase npwwfn. ',ch10,&
& ' npwepG0 = ',Ep%npwepG0,' npwvec = ',Ep%npwvec
 call assert(ltest,msg,__FILE__,__LINE__)
 !
 ! === Setup of the FFT mesh used for the oscillator matrix elements ===
 ! * ngfft_gw(7:18) is the same as Dtset%ngfft(7:18), initialized before entering setup_screening. 
 !   Here we just redefine ngfft_gw(1:6) according to the following options:
 !
 !    method==0 ==> FFT grid read from __fft.in__ (only for debugging purpose)
 !    method==1 ==> normal FFT grid 
 !    method==2 ==> slightly augmented FFT grid to calculate exactly rho_tw_g (see setmesh.F90)
 !    method==3 ==> doubled FFT grid, to treat exactly the convolution defining the density,
 !      Useful in sigma if ppmodel=[2,3,4] since rho(G-Gp) or to calculate matrix elements of v_Hxc.
 !
 !    enforce_sym==1 ==> enforce a direct space FFT mesh compatible with all symmetries operation
 !    enforce_sym==0 ==> Find the smallest FFT grid compatibile with the library, do not care about symmetries
 !
 ngfft_gw(1:18)=Dtset%ngfft(1:18) ; method=2
 if (Dtset%fftgw==00 .or. Dtset%fftgw==01) method=0
 if (Dtset%fftgw==10 .or. Dtset%fftgw==11) method=1
 if (Dtset%fftgw==20 .or. Dtset%fftgw==21) method=2
 if (Dtset%fftgw==30 .or. Dtset%fftgw==31) method=3
 enforce_sym=MOD(Dtset%fftgw,10) 

 !call setmesh(gmet,gvec_p,ngfft_gw,Ep%npwvec,Ep%npwe,Ep%npwwfn,nfftgw_tot,method,Ep%mG0,Cryst,enforce_sym)
 call setmesh(gmet,gvec_p,ngfft_gw,Ep%npwvec,Ep%npwepG0,Ep%npwwfn,nfftgw_tot,method,Ep%mG0,Cryst,enforce_sym)

 ! === Create structure describing the G-sphere used for chi0/espilon and Wfns ===
 ! * The cutoff is >= ecuteps to allow for umklapp
 ! MG, Fabien, I modified setup_G_rotation to speed up the loops, 
 ! sincerely I dont like only_one_kpt because some entries in Gsphere are not correctly filled!
 only_one_kpt=(Kmesh%nbz==1) 
 call nullify_Gvectors(Gsph_epsG0)
 call nullify_Gvectors(Gsph_wfn  )
 call init_Gvectors_type(only_one_kpt,Gsph_epsG0,Cryst,Ep%npwepG0,gvec_p,gmet,gprimd)
 call init_Gvectors_type(only_one_kpt,Gsph_wfn  ,Cryst,Ep%npwvec ,gvec_p,gmet,gprimd)
 deallocate(gvec_p)

 ! FIXME this wont work if nqptdm/=0
 call setup_coulombian(Dtset,Gsph_epsG0,Qmesh,Kmesh,Ep%npwe,Cryst%rprimd,ngfftf,MPI_enreg,Vcp)

 ! Value of scissor energy
 Ep%soenergy=Dtset%soenergy

 ! === Define the frequency mesh for epsilon according to the method used ===
 Ep%nomegaei=1 
 Ep%nomegaer=1 ; if (is_static) Ep%nomegaer=0
 Ep%omegaermax=zero

 if (Ep%plasmon_pole_model.and.Dtset%nfreqre==1.and.Dtset%nfreqim==0) then
  ! === For ppmodels 2,3,4, only omega=0 is needed ===
  Ep%nomegaer=1 ; Ep%nomegaei=0
  write(msg,'(7a)')ch10,&
&  ' The inverse dielectric matrix will be calculated on zero frequency only',ch10,&
&  ' please note that the calculated epsilon^-1 cannot be used ',ch10,&
&  ' to calculate QP corrections using plasmonpole model 1',ch10
  call wrtout(std_out,msg,'COLL') 
  call wrtout(ab_out,msg,'COLL')
 end if 
 !
 ! === Max number of omega along the imaginary axis ===
 if (Ep%analytic_continuation.or.Ep%contour_deformation) then
  Ep%nomegaei=Dtset%nfreqim
  if (Ep%nomegaei==0) then 
   Ep%nomegaei=NOMEGAGAUSS
   write(msg,'(2a,i5)')ch10,&
&   ' Number of imaginary frequencies set to default = ',NOMEGAGAUSS
   MSG_WARNING(msg)
  end if
 end if

 ! === Range and total number of real frequencies ===
 if (Ep%contour_deformation) then
  Ep%nomegaer=Dtset%nfreqre ; Ep%omegaermax=Dtset%freqremax
  if (Ep%nomegaer==0) then 
   Ep%nomegaer=NOMEGAREAL
   write(msg,'(2a,i5)')ch10,&
&   ' Number of real frequencies set to default = ',NOMEGAREAL
   MSG_WARNING(msg)
  end if 
  if (Ep%omegaermax<0.1d-4) then 
   Ep%omegaermax=OMEGAERMAX
   write(msg,'(2a,f8.4)')ch10,&
&   '  Max real frequencies set to default [Ha] = ',OMEGAERMAX
   MSG_WARNING(msg)
  end if 
 end if

 Ep%nomega=Ep%nomegaer+Ep%nomegaei ! Total number of frequencies.

 ! === Setup of spectral method ===
 Ep%spmeth  =Dtset%spmeth 
 Ep%nomegasf=Dtset%nomegasf 
 Ep%spsmear =Dtset%spbroad

 if (Ep%spmeth/=0) then 
  write(msg,'(2a,i3,2a,i8)')ch10,&
&  ' setup_screening : using spectral method = ',Ep%spmeth,ch10,&
&  '  Number of frequencies for imaginary part= ',Ep%nomegasf
  call wrtout(std_out,msg,'COLL')
  if (Ep%spmeth==2) then 
   write(msg,'(a,f8.5,a)')' gaussian broadening = ',Ep%spsmear*Ha_eV,' [eV]'
   call wrtout(std_out,msg,'COLL')
  end if  
 end if

 Ep%nI=1 ; Ep%nJ=1 
 if (Dtset%nspinor==2) then 
  if (Dtset%usepaw==1.and.Dtset%pawspnorb>0) then
   Ep%nI=1 ; Ep%nJ=4 
  end if
  ! For spin-spin interaction
  ! Ep%nI=4 ; Ep%nJ=4 
  if (Ep%npwepG0/=Ep%npwe) STOP "If spinor npwepG0 must be == npwe"
  if (Ep%symchi/=0) STOP "symchi/0 and spinor not available"
 end if
 !
 ! === Enable the calculations of chi0 on user-specified q-points ===
 Ep%nqibz=Qmesh%nibz
 allocate(Ep%qibz(3,Ep%nqibz))
 Ep%qibz(:,:)=Qmesh%ibz(:,:)

 Ep%nqcalc=Ep%nqibz
 if (Dtset%nqptdm>0) Ep%nqcalc=Dtset%nqptdm

 allocate(Ep%qcalc(3,Ep%nqcalc))
 if (Ep%nqcalc/=Ep%nqibz) then
  write(msg,'(6a)')ch10,&
&  ' Dielectric matrix will be calculated only for some ',ch10,&
&  ' selected q points provided by the user through the input variables ',ch10,&
&  ' nqptdm and qptdm'
  call wrtout(std_out,msg,'COLL')  
  call wrtout(ab_out,msg,'COLL')
  ltest=(Ep%nqcalc<=Qmesh%nibz)
  write(msg,'(a)')'nqptdm should not exceed the number of q points in the IBZ'
  call assert(ltest,msg,__FILE__,__LINE__)
  ! Size of 2-dimension of Dtset%qptdm might be larger
  Ep%qcalc(:,:)=Dtset%qptdm(:,1:Ep%nqcalc)
  ! * Check that the provided q-points are correct.
  do iq=1,Ep%nqcalc
   found=.FALSE.
   do iqp=1,Qmesh%nibz
    qtmp(:)=Ep%qcalc(:,iq)-Qmesh%ibz(:,iqp) 
    found=(normv(qtmp,gmet,'G')<GW_TOLQ) 
    if (found) EXIT
   end do
   write(msg,'(a)')'One or more points specified by Dtset%qptdm do not satisfy q=k1-k2'
   call assert(found,msg,__FILE__,__LINE__)
  end do 
 else 
  Ep%qcalc(:,:)=Ep%qibz(:,:)
 end if 

 ! To write the SCR header correctly, with heads and wings, we have 
 ! to make sure that q==0, if present, is the first q-point in the list.
 has_q0=(ANY(normv(Ep%qcalc(:,:),gmet,'G')<GW_TOLQ0))
 if (has_q0.and.normv(Ep%qcalc(:,1),gmet,'G')>=GW_TOLQ0) then
  write(msg,'(6a)')ch10,&
&  ' The list of q-points to be calculated contains the Gamma point, ',ch10,&
&  ' however Gamma is not the first point in the list. ' ,ch10,&
&  ' Please, change your input file accordingly. '
  MSG_ERROR(msg)
 end if

 ! === Initialize the band structure datatype ===
 ! * Copy KSS energies and occupations up to Sp%nbnds==Dtset%nband(:)
 ! TODO Recheck symmorphy and inversion

 bantot=SUM(Dtset%nband(1:Dtset%nkpt*Dtset%nsppol))

 allocate(doccde(bantot),eigen(bantot),occfact(bantot))
 doccde(:)=zero ; eigen(:)=zero ; occfact(:)=zero 

 jj=0 ; ibtot=0
 do isppol=1,Dtset%nsppol
  do ikibz=1,Dtset%nkpt
   do ib=1,Hdr_kss%nband(ikibz+Dtset%nkpt*(isppol-1))
    ibtot=ibtot+1
    if (ib<=Ep%nbnds) then 
     jj=jj+1
     occfact(jj)=Hdr_kss%occ(ibtot)
     eigen  (jj)=energies_p(ib,ikibz,isppol)
    end if
   end do
  end do
 end do
 deallocate(energies_p)

 ! Make sure that Dtset%wtk==Kmesh%wt due to the dirty treatment of 
 ! the symmetry operations in the old GW code (symmorphy and inversion) 
 ltest=(ALL(ABS(Dtset%wtk(1:Kmesh%nibz)-Kmesh%wt(1:Kmesh%nibz))<tol6))
 call assert(ltest,'Mismatch between Dtset%wtk and Kmesh%wt',__FILE__,__LINE__)

 allocate(npwarr(Hdr_kss%nkpt)) ; npwarr(:)=Ep%npwwfn 

 call bstruct_init(bantot,KS_BSt,Dtset%nelect,doccde,eigen,Dtset%istwfk,Kmesh%ibz,Dtset%nband,&
& Kmesh%nibz,npwarr,Dtset%nsppol,Dtset%nspinor,Dtset%tphysel,Dtset%tsmear,Dtset%occopt,occfact,Kmesh%wt) 

 ! TODO modify outkss in order to calculate the eigenvalues also if NSCF calculation.
 ! this fails simply because in case of NSCF occ  are zero
 !ltest=(ALL(ABS(occfact-KS_BSt%occ)<1.d-2)) 
 !call assert(ltest,'difference in occfact')
 !write(*,*)MAXVAL(ABS(occfact(:)-KS_BSt%occ(:))) 

 deallocate(doccde,eigen,npwarr)

 ! === Initialize abinit header for the screening part ===
 pertcase_=0
 call hdr_init(KS_BSt,codvsn,Dtset,Hdr_out,Pawtab,pertcase_,Psps)

 ! === Get Pawrhoij from the header of the KSS file ===
 allocate(Pawrhoij(Cryst%natom*Dtset%usepaw))
 if (Dtset%usepaw==1) then
  allocate(nlmn(Cryst%ntypat))
  do itypat=1,Cryst%ntypat
   nlmn(itypat)=Pawtab(itypat)%lmn_size
  end do
  call rhoij_alloc(1,nlmn,Dtset%nspden,Dtset%nsppol,Pawrhoij,Cryst%typat)
  deallocate(nlmn)
  call rhoij_copy(Hdr_kss%Pawrhoij,Pawrhoij)
 end if

 call hdr_update(bantot,1.0d20,1.0d20,Hdr_out,Cryst%natom,1.0d20,&
& Cryst%rprimd,occfact,Pawrhoij,Dtset%usepaw,Cryst%xred)

! call hdr_update(bantot,1.0d20,1.0d20,Hdr_out,Cryst%natom,1.0d20,&
!& Cryst%rprimd,KS_BSt%occ,Pawrhoij,Dtset%usepaw,Cryst%xred)

 ! This is just to do a check, the file format is wrong!
 call hdr_check(1002,1002,Hdr_out,Hdr_kss,'COLL',restart,restartpaw)

 deallocate(occfact)
 call rhoij_free(Pawrhoij) ; deallocate(Pawrhoij)

 ! === Final compatibility tests ===
 ltest=ALL(KS_BSt%istwfk==1) 
 call assert(ltest,'istwfk must be 1',__FILE__,__LINE__)
 ltest=(KS_BSt%mband==Ep%nbnds.and.ALL(KS_BSt%nband==Ep%nbnds))
 call assert(ltest,'BUG in definition of KS_BSt%nband',__FILE__,__LINE__)

 DBG_EXIT('COLL')

end subroutine setup_screening
!!***
